@@ -48,7 +48,9 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 using namespace std;
 
-
+namespace {
+	PlayerInfo *currentPlayer;
+}
 
 // Completely clear all loaded information, to prepare for loading a file or
 // creating a new pilot.
@@ -305,6 +307,8 @@ void PlayerInfo::Load(const string &path)
 	// will count as non-depreciated.
 	if(!depreciation.IsLoaded())
 		depreciation.Init(ships, date.DaysSinceEpoch());
+	
+	currentPlayer = this;
 }
 
 
@@ -361,7 +365,7 @@ void PlayerInfo::Save() const
 	Save(filePath);
 }
 
-
+PlayerInfo *PlayerInfo::CurrentPlayer() { return currentPlayer; }
 
 // Get the base file name for the player, without the ".txt" extension. This
 // will usually be "<first> <last>", but may be different if multiple players
@@ -1113,6 +1117,8 @@ int64_t PlayerInfo::GetBasis(const string &commodity, int tons) const
 // ships, check for mission completion, and apply fines for contraband.
 void PlayerInfo::Land(UI *ui)
 {
+	currentPlayer = this;
+	
 	// This can only be done while landed.
 	if(!system || !planet)
 		return;
@@ -1235,6 +1241,8 @@ void PlayerInfo::Land(UI *ui)
 // which case a message will be returned.
 bool PlayerInfo::TakeOff(UI *ui)
 {
+	currentPlayer = this;
+	
 	// This can only be done while landed.
 	if(!system || !planet)
 		return false;
