@@ -15,13 +15,18 @@ int maine(int argc, char *argv[]);
 #include "endless-sky/source/Angle.h"
 #include "endless-sky/source/DataNode.h"
 #include "endless-sky/source/DataFile.h"
+#include "endless-sky/source/DataWriter.h"
 #include "endless-sky/source/GameData.h"
+#include "endless-sky/source/Government.h"
 #include "endless-sky/source/Outfit.h"
+#include "endless-sky/source/Planet.h"
 #include "endless-sky/source/PlayerInfo.h"
 #include "endless-sky/source/Point.h"
 #include "endless-sky/source/Random.h"
+#include "endless-sky/source/SavedGame.h"
 #include "endless-sky/source/Set.h"
 #include "endless-sky/source/Ship.h"
+#include "endless-sky/source/System.h"
 #include "endless-sky/tests/include/datanode-factory.h"
 
 #define STRINGIFY(x) #x
@@ -104,6 +109,28 @@ PYBIND11_MODULE(bindings, m) {
             return py::make_iterator(f.begin(), f.end());
         }, py::keep_alive<0, 1>()); // TODO is this keep_alive policy right?
 
+    // source/DataWriter
+    py::class_<DataWriter>(m, "DataWriter")
+        .def(py::init<std::string&>())
+        .def("Write", [](DataWriter &w, const DataNode &node) {
+            return w.Write(node);
+        })
+        .def("Write", [](DataWriter &w) {
+            return w.Write();
+        })
+
+        .def("BeginChild", &DataWriter::BeginChild)
+        .def("WriteComment", &DataWriter::WriteComment)
+        .def("WriteToken", [](DataWriter &w, const double &a) {
+            return w.WriteToken(a);
+        })
+        .def("WriteToken", [](DataWriter &w, const int &a) {
+            return w.WriteToken(a);
+        })
+        .def("WriteToken", [](DataWriter &w, const std::string &a) {
+            return w.WriteToken(a);
+        });
+
     // source/DataNode
     py::class_<DataNode>(m, "DataNode")
         .def(py::init<DataNode*>())
@@ -141,7 +168,47 @@ PYBIND11_MODULE(bindings, m) {
             return GameData::BeginLoad(cstrs.data());
         })
 	.def_static("CheckReferences", &GameData::CheckReferences)
-	.def_static("Ships", &GameData::Ships);
+	.def_static("Ships", &GameData::Ships)
+	.def_static("Governments", &GameData::Governments)
+	.def_static("Outfits", &GameData::Outfits)
+	.def_static("Planets", &GameData::Planets)
+	.def_static("Systems", &GameData::Systems);
+
+    // source/Government
+    py::class_<Government>(m, "Government")
+        .def(py::init<>())
+        .def("Load", &Government::Load)
+        .def("GetName", &Government::GetName)
+        .def("SetName", &Government::SetName)
+        .def("GetTrueName", &Government::GetTrueName)
+        .def("GetSwizzle", &Government::GetSwizzle)
+        .def("GetColor", &Government::GetColor)
+
+        .def("AttitudeToward", &Government::AttitudeToward)
+        .def("InitialPlayerReputation", &Government::InitialPlayerReputation)
+        .def("PenaltyFor", &Government::PenaltyFor)
+        .def("GetBribeFraction", &Government::GetBribeFraction)
+        .def("GetFineFraction", &Government::GetFineFraction)
+
+        .def("CanEnforce", py::overload_cast<const System*>(&Government::CanEnforce, py::const_))
+        .def("CanEnforce", py::overload_cast<const Planet*>(&Government::CanEnforce, py::const_))
+        .def("DeathSentence", &Government::DeathSentence)
+
+        .def("GetHail", &Government::GetHail)
+        .def("Language", &Government::Language)
+        .def("RaidFleet", &Government::RaidFleet)
+
+        .def("IsEnemy", py::overload_cast<const Government*>(&Government::IsEnemy, py::const_))
+        .def("IsEnemy", py::overload_cast<>(&Government::IsEnemy, py::const_ ))
+        .def("IsPlayer", &Government::IsPlayer)
+        .def("Offend", &Government::Offend)
+        .def("Bribe", &Government::Bribe)
+        .def("GetHail", &Government::GetHail)
+        .def("Fine", &Government::Fine)
+        .def("Reputation", &Government::Reputation)
+        .def("SetReputation", &Government::SetReputation)
+        .def("CrewAttack", &Government::CrewAttack)
+        .def("CrewDefense", &Government::CrewDefense);
 
     // source/Outfit
     py::class_<Outfit>(m, "Outfit")
@@ -150,6 +217,54 @@ PYBIND11_MODULE(bindings, m) {
         .def("Name", &Outfit::Name)
         .def("Attributes", &Outfit::Attributes);
 
+    // source/Planet
+    py::class_<Planet>(m, "Planet")
+        // TODO add a constructor that takes data?
+        .def("Load", &Planet::Load)
+        .def("IsValid", &Planet::IsValid)
+        .def("Name", &Planet::Name)
+        .def("SetName", &Planet::SetName)
+        .def("TrueName", &Planet::TrueName)
+        .def("Description", &Planet::Description)
+//        .def("Landscape", &Planet::Landscape)
+        .def("MusicName", &Planet::MusicName)
+        .def("Attributes", &Planet::Attributes)
+        .def("Noun", &Planet::Noun)
+
+        .def("HasSpaceport", &Planet::HasSpaceport)
+        .def("SpaceportDescription", &Planet::SpaceportDescription)
+        .def("HasShipyard", &Planet::HasShipyard)
+        .def("Shipyard", &Planet::Shipyard)
+        .def("Outfitter", &Planet::Outfitter)
+
+//        .def("Government", &Planet::Government)
+        .def("RequiredReputation", &Planet::RequiredReputation)
+        .def("GetBribeFraction", &Planet::GetBribeFraction)
+        .def("Security", &Planet::Security)
+
+        .def("GetSystem", &Planet::GetSystem)
+        .def("IsInSystem", &Planet::IsInSystem)
+        .def("SetSystem", &Planet::SetSystem)
+        .def("RemoveSystem", &Planet::RemoveSystem)
+
+        .def("IsWormhole", &Planet::IsWormhole)
+        .def("WormholeSource", &Planet::WormholeSource)
+        .def("WormholeDestination", &Planet::WormholeDestination)
+        .def("WormholeSystems", &Planet::WormholeSystems)
+
+        .def("IsAccessible", &Planet::IsAccessible)
+        .def("IsUnrestricted", &Planet::IsUnrestricted)
+
+        .def("HasFuelFor", &Planet::HasFuelFor)
+        .def("CanLand", py::overload_cast<>(&Planet::CanLand, py::const_))
+        .def("CanLand", py::overload_cast<const Ship&>(&Planet::CanLand, py::const_))
+        .def("CanUseServices", &Planet::CanUseServices)
+        .def("Bribe", &Planet::Bribe)
+
+        .def("DemandTribute", &Planet::DemandTribute)
+        .def("DeployDefense", &Planet::DeployDefense)
+        .def("ResetDefense", &Planet::ResetDefense);
+
     // source/PlayerInfo
     py::class_<PlayerInfo>(m, "PlayerInfo")
         .def("Ships", &PlayerInfo::Ships)
@@ -167,8 +282,34 @@ PYBIND11_MODULE(bindings, m) {
     m.def("RandomInt", py::overload_cast<uint32_t>(&Random::Int));
 
     // source/Set
-    std::string a = std::string("Ship");
-    declare_set<Ship>(m, a);
+    std::string shipString = std::string("Ship");
+    declare_set<Ship>(m, shipString);
+    std::string governmentString = std::string("Government");
+    declare_set<Government>(m, governmentString);
+    std::string outfitString = std::string("Outfit");
+    declare_set<Outfit>(m, outfitString);
+    std::string planetString = std::string("Planet");
+    declare_set<Planet>(m, planetString);
+    std::string systemString = std::string("System");
+    declare_set<System>(m, systemString);
+
+    // source/SavedGame
+    py::class_<SavedGame, std::shared_ptr<SavedGame>>(m, "SavedGame")
+        .def(py::init<>())
+        .def(py::init<const std::string&>())
+        .def("Load", &SavedGame::Load)
+        .def("Path", &SavedGame::Path)
+        .def("IsLoaded", &SavedGame::IsLoaded)
+        .def("Clear", &SavedGame::Clear)
+        .def("Name", &SavedGame::Name)
+        .def("Credits", &SavedGame::Credits)
+
+        .def("GetSystem", &SavedGame::GetSystem)
+        .def("GetPlanet", &SavedGame::GetPlanet)
+        .def("GetPlayTime", &SavedGame::GetPlayTime)
+
+//        .def("ShipSprite", &SavedGame::ShipSprite)
+        .def("ShipName", &SavedGame::ShipName);
 
     // source/Ship
     py::class_<Ship, std::shared_ptr<Ship>>(m, "Ship")
@@ -201,10 +342,10 @@ PYBIND11_MODULE(bindings, m) {
         .def("SetPosition", &Ship::SetPosition)
         .def("Place", &Ship::Place)
         .def("SetName", &Ship::SetName)
-//        .def("SetSystem", &Ship::SetSystem)
-//        .def("SetPlanet", &Ship::SetPlanet)
-//        .def("SetGovernment", &Ship::SetGovernment)
-//        .def("SetIsSpecial", &Ship::SetIsSpecial)
+        .def("SetSystem", &Ship::SetSystem)
+        .def("SetPlanet", &Ship::SetPlanet)
+        .def("SetGovernment", &Ship::SetGovernment)
+        .def("SetIsSpecial", &Ship::SetIsSpecial)
         .def("IsSpecial", &Ship::IsSpecial)
 
         .def("SetIsYours", &Ship::SetIsYours)
@@ -224,16 +365,16 @@ PYBIND11_MODULE(bindings, m) {
 //        .def("Move", &Ship::Move)
 //        .def("DoGeneration", &Ship::DoGeneration)
 //        .def("Launch", &Ship::Launch)
-//        .def("Board", &Ship::Board)
-//        .def("Scan", &Ship::Scan)
-//        .def("CargoScanFraction", &Ship::CargoScanFraction)
-//        .def("OutfitScanFraction", &Ship::OutfitScanFraction)
+        .def("Board", &Ship::Board)
+        .def("Scan", &Ship::Scan)
+        .def("CargoScanFraction", &Ship::CargoScanFraction)
+        .def("OutfitScanFraction", &Ship::OutfitScanFraction)
 
 //        .def("Fire", &Ship::Fire)
 //        .def("FireAntiMissile", &Ship::FireAntiMissile)
 
-//        .def("GetSystem", &Ship::GetSystem)
-//        .def("GetPlanet", &Ship::GetPlanet)
+        .def("GetSystem", &Ship::GetSystem)
+        .def("GetPlanet", &Ship::GetPlanet)
 
 	.def("IsCapturable", &Ship::IsCapturable)
 	.def("IsTargetable", &Ship::IsTargetable)
@@ -276,7 +417,7 @@ PYBIND11_MODULE(bindings, m) {
         .def("Health", &Ship::Health)
         .def("DisabledHull", &Ship::DisabledHull)
         .def("JumpsRemaining", &Ship::JumpsRemaining)
-//        .def("JumpFuel", &Ship::JumpFuel)
+        .def("JumpFuel", &Ship::JumpFuel)
         .def("JumpRange", &Ship::JumpRange)
         .def("HyperdriveFuel", &Ship::HyperdriveFuel)
         .def("JumpDriveFuel", &Ship::JumpDriveFuel)
@@ -306,6 +447,57 @@ PYBIND11_MODULE(bindings, m) {
 
         // Custom helpers could go here (in lowercase)
 
+    // source/System
+    py::class_<System>(m, "System")
+        .def("Load", &System::Load)
+        .def("UpdateSystem", &System::UpdateSystem)
+
+        .def("Link", &System::Link)
+        .def("Unlink", &System::Unlink)
+
+        .def("IsValid", &System::IsValid)
+        .def("Name", &System::Name)
+        .def("SetName", &System::SetName)
+        .def("Position", &System::Position)
+        .def("Government", &System::GetGovernment)
+        .def("MusicName", &System::MusicName)
+
+        .def("Attributes", &System::Attributes)
+
+        .def("Links", &System::Links)
+        .def("JumpNeighbors", &System::JumpNeighbors)
+        .def("Hidden", &System::Hidden)
+        .def("ExtraHyperArrivalDistance", &System::ExtraHyperArrivalDistance)
+        .def("ExtraJumpArrivalDistance", &System::ExtraJumpArrivalDistance)
+        .def("VisibleNeighbors", &System::VisibleNeighbors)
+
+        .def("SetDate", &System::SetDate)
+//        .def("Objects", &System::Objects)
+//        .def("FindStellar", &System::SetDate)
+        .def("HabitableZone", &System::HabitableZone)
+        .def("AsteroidBelt", &System::AsteroidBelt)
+        .def("JumpRange", &System::JumpRange)
+        .def("SolarPower", &System::SolarPower)
+        .def("SolarWind", &System::SolarWind)
+        .def("IsInhabited", &System::IsInhabited)
+        .def("HasFuelFor", &System::HasFuelFor)
+        .def("HasShipyard", &System::HasShipyard)
+        .def("HasOutfitter", &System::HasOutfitter)
+
+        .def("Asteroids", &System::Asteroids)
+//        .def("Haze", &System::Haze)
+
+        .def("Trade", &System::Trade)
+        .def("HasTrade", &System::HasTrade)
+
+        .def("StepEconomy", &System::StepEconomy)
+        .def("SetSupply", &System::SetSupply)
+        .def("Supply", &System::Supply)
+        .def("Exports", &System::Exports)
+
+//        .def("Fleets", &System::Fleets)
+//        .def("Hazards", &System::Hazards)
+        .def("Danger", &System::Danger);
 
     // source/main
     m.def("main", [](std::vector<std::string> argVec) {
